@@ -26,7 +26,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-create-layout.el,v 1.27 2003/12/28 15:28:57 berndl Exp $
+;; $Id: ecb-create-layout.el,v 1.29 2004/05/06 09:02:08 berndl Exp $
 
 ;;; Commentary:
 ;;
@@ -45,6 +45,7 @@
 
 (require 'ecb-mode-line)
 (require 'ecb-util)
+(require 'ecb-compilation)
 
 ;; XEmacs stuff
 (silentcomp-defvar vertical-divider-map)
@@ -233,8 +234,8 @@
   "Ensure calling `ecb-create-layout-cancel' during deleting the
 layout-creation frame."
   (let ((frame (or (ad-get-arg 0) (selected-frame))))
-    (when (string= (ecb-frame-parameter frame 'name)
-                   ecb-create-layout-frame-name)
+    (when (ecb-string= (ecb-frame-parameter frame 'name)
+                       ecb-create-layout-frame-name)
       (ecb-create-layout-cancel))))
 
 (defun ecb-create-layout-frame-ok ()
@@ -249,7 +250,8 @@ layout-creation frame."
   (interactive)
   (when (ecb-create-layout-frame-ok)
     (ecb-create-layout-clear-all (interactive-p))
-    (message "ECB Layout Creation canceled - the layout is not saved!")))
+    (message "ECB Layout Creation canceled - the layout is not saved!")
+    (ecb-activate)))
 
 (defun ecb-create-layout-clear-all (&optional delete-frame)
   "Resets all stuff to state before `ecb-create-new-layout' was called. If
@@ -306,7 +308,8 @@ DELETE-FRAME is not nil then the new created frame will be deleted and the
               (ecb-create-layout-save-layout)
             ;; clean the layout creation stuff
             (ecb-create-layout-clear-all delete-frame)
-            (message "ECB Layout Creation finished.")))
+            (message "ECB Layout Creation finished.")
+            (ecb-activate)))
       (ecb-error "You must give every ECB-tree-window a type (use C-t)!"))))
 
 
@@ -400,12 +403,12 @@ DELETE-FRAME is not nil then the new created frame will be deleted and the
              (member type ecb-create-layout-all-buf-types))
     (add-to-list 'ecb-create-layout-buf-types type)
     (setq ecb-create-layout-buf-types
-          (sort ecb-create-layout-buf-types 'string-lessp))))
+          (sort ecb-create-layout-buf-types 'ecb-string<))))
 
 (defun ecb-create-layout-remove-from-buf-type (type)
   (when (stringp type)
     (setq ecb-create-layout-buf-types
-          (sort (delete type ecb-create-layout-buf-types) 'string-lessp))))
+          (sort (delete type ecb-create-layout-buf-types) 'ecb-string<))))
 
 (defun ecb-create-layout-buffer-type ()
   (get-text-property (point-min) 'ecb-create-layout-type))
@@ -445,7 +448,10 @@ DELETE-FRAME is not nil then the new created frame will be deleted and the
                                           "Insert the buffer type"))))
       ;; removing the new buffer type from the available-list
       (ecb-create-layout-remove-from-buf-type new-type)
+      ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Fix this - it seems not
+      ;; work anymore!!!
       (ecb-mode-line-set (buffer-name (current-buffer))
+                         (selected-frame)
                          (concat "ECB " new-type) nil t)
       ;; setting the new buffer type in the buffer itself
       (ecb-create-layout-set-buffer-type new-type)
@@ -479,15 +485,15 @@ DELETE-FRAME is not nil then the new created frame will be deleted and the
             (ecb-query-string "Split method:"
                               '("at-point" "half")
                               "Insert a fraction between 0.1 and 0.9"))
-           (fraction (cond ((string= split-method "at-point")
+           (fraction (cond ((ecb-string= split-method "at-point")
                             nil)
-                           ((string= split-method "half")
+                           ((ecb-string= split-method "half")
                             0.5)
                            ((floatp (string-to-number split-method))
                             (string-to-number split-method))
                            (t 0.5)))
            (real-split-factor
-            (if (string= split-type "horizontal")
+            (if (ecb-string= split-type "horizontal")
                 (ecb-create-layout-split-hor fraction)
               (ecb-create-layout-split-ver fraction))))
       ;; creating new fitting buffers
@@ -652,7 +658,10 @@ never selects the edit-window."
   (setq mode-name "ECB Create-Layout")
   (use-local-map ecb-create-layout-mode-map)
   (make-variable-buffer-local 'buffer-read-only)
-  (ecb-mode-line-set (buffer-name (current-buffer)) "" nil t)
+  ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Scheint nicht mehr zu
+  ;; funktionieren.
+  (ecb-mode-line-set (buffer-name (current-buffer))
+                     (selected-frame) "" nil t)
   (setq buffer-read-only t))
 
 (defun ecb-create-layout-init-layout (&optional new)
@@ -679,7 +688,10 @@ never selects the edit-window."
                 ecb-create-layout-help-text-top
               ecb-create-layout-help-text-left-right)))
   (setq ecb-create-layout-edit-window (selected-window))
-  (ecb-mode-line-set (buffer-name (current-buffer)) "   ECB edit-window" nil t)
+  ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Scheint nicht mehr zu
+  ;; funktionieren.
+  (ecb-mode-line-set (buffer-name (current-buffer))
+                     (selected-frame) "   ECB edit-window" nil t)
   ;; The edit window must not be dedicated
   (set-window-dedicated-p (selected-window) nil)
   ;; we set the buffer for the (currently unsplitted) ECB-window
@@ -735,11 +747,14 @@ never selects the edit-window."
                        (menu-bar-lines . 0))))))
          
 
-
+;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Wir mssen ev. ECB vorher
+;; deaktivieren, da sonst ein 2. ECB-menu entsteht. Beim C-c oder C-q eben
+;; dann wieder aktivieren.
 (defun ecb-create-new-layout ()
   "Start interactively layout creating."
   (interactive)
 
+  (ecb-deactivate)
   (ecb-create-layout-initilize)
 
   ;; before- and after make frame stuff
@@ -799,7 +814,7 @@ unbound."
                                (mapcar (function (lambda (elem)
                                                    (car elem)))
                                        ecb-buildin-layouts))
-               'string<))
+               'ecb-string<))
         (layout-name nil))
     (if (= (length new-layout-list) 0)
         (ecb-error "There are no layouts to delete!")
