@@ -26,7 +26,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-help.el,v 1.103 2004/01/21 17:17:46 berndl Exp $
+;; $Id: ecb-help.el,v 1.108 2004/08/27 15:42:22 berndl Exp $
 
 ;;; Commentary:
 ;;
@@ -227,7 +227,16 @@ This command generates in the edit-window a problem-report which contains
 already the current values of all ECB options, the current backtrace-buffer if
 there is any and the current message-buffer. You will be asked for a
 problem-report subject and then you must insert a description of the problem.
-Please describe the problem as detailed as possible!"
+Please describe the problem as detailed as possible!
+
+*IMPORTANT*: Cause of extra appearance of SPAM in the mailing-lists,
+SourceForge has changed its policy: Now it is only possible to post to the
+mailing-list for users who have subscribed this mailing-list. So please be
+aware you will not be able to send comments, bug reports and improvement
+suggestions before you have subscribed the ECB-mailing-list. See the section
+\"Mailing-list\" at the ECB-website at http://ecb.sourceforge.net how to do
+this."
+
   (interactive)
   (when (or ecb-minor-mode
             (y-or-n-p "ECB should be active when submitting a problem-report. Force report? "))
@@ -300,7 +309,32 @@ a backtrace-buffer and inserts the contents of that."
     (let* ((messages-buffer 
 	    (get-buffer
 	     (if ecb-running-xemacs " *Message-Log*" "*Messages*")))
-	   (backtrace-buffer (get-buffer "*Backtrace*")))
+	   (backtrace-buffer (get-buffer "*Backtrace*"))
+           (tag-dump-buffer (get-buffer "*ecb-tag-dump*")))
+
+      ;;insert the contents of the tag-dump buffer if it is there. 
+      (insert "\n\n-----------------------------------------------------\n")
+      (if tag-dump-buffer
+          (progn
+            (insert "The contents of the *ecb-tag-dump* buffer were\n\n")
+	    (insert-buffer tag-dump-buffer)
+            ;; we must force the mark
+	    (goto-char (mark t))
+            (insert "\nEnd Insert *ecb-tag-dump* buffer" ))
+        (insert "There was no *ecb-tag-dump* buffer" ))
+      (insert "\n-----------------------------------------------------\n\n")
+
+      ;;insert the contents of the trace-output buffer if it is there. 
+;;       (insert "\n\n-----------------------------------------------------\n")
+;;       (if tag-dump-buffer
+;;           (progn
+;;             (insert "The contents of the *ecb-tag-dump* buffer were\n\n")
+;; 	    (insert-buffer tag-dump-buffer)
+;;             ;; we must force the mark
+;; 	    (goto-char (mark t))
+;;             (insert "\nEnd Insert *ecb-tag-dump* buffer" ))
+;;         (insert "There was no *ecb-tag-dump* buffer" ))
+;;       (insert "\n-----------------------------------------------------\n\n")
 
       ;;insert the contents of the backtrace buffer if it is there. 
       (insert "\n\n-----------------------------------------------------\n")
@@ -359,8 +393,8 @@ could be interesting for support."
                                     ,(if (boundp 'ediff-quit-hook)
                                          'ediff-quit-hook)))
                           (function (lambda (l r)
-                                      (string< (symbol-name l)
-                                               (symbol-name r))))))
+                                      (ecb-string< (symbol-name l)
+                                                   (symbol-name r))))))
         (semantic-vars (sort (delete nil
                                      `(semantic-after-toplevel-cache-change-hook
                                        semantic-after-partial-cache-change-hook
@@ -370,8 +404,8 @@ could be interesting for support."
                                        semantic-uml-colon-string
                                        semantic-orphaned-member-metaparent-type))
                              (function (lambda (l r)
-                                         (string< (symbol-name l)
-                                                  (symbol-name r))))))
+                                         (ecb-string< (symbol-name l)
+                                                      (symbol-name r))))))
         (speedbar-vars (sort '(speedbar-dynamic-tags-function-list
                                speedbar-tag-hierarchy-method
                                speedbar-tag-group-name-minimum-length
@@ -381,14 +415,14 @@ could be interesting for support."
                                speedbar-fetch-etags-arguments
                                speedbar-fetch-etags-parse-list)
                              (function (lambda (l r)
-                                         (string< (symbol-name l)
-                                                  (symbol-name r))))))
+                                         (ecb-string< (symbol-name l)
+                                                      (symbol-name r))))))
         (ecb-options (mapcar
                       'intern
                       (sort
                        (let (completion-ignore-case)
                          (all-completions "ecb-" obarray 'user-variable-p))
-                       'string-lessp)))
+                       'ecb-string<)))
         (ecb-internal-vars (sort '(ecb-path-selected-directory
                                    ecb-path-selected-source
                                    ecb-use-semantic-grouping
@@ -402,10 +436,10 @@ could be interesting for support."
                                    ecb-windows-hidden
                                    ecb-toggle-layout-state
                                    ecb-current-maximized-ecb-buffer-name
-                                   ecb-tree-buffers-of-current-layout)
+                                   ecb-special-ecb-buffers-of-current-layout)
                                  (function (lambda (l r)
-                                             (string< (symbol-name l)
-                                                      (symbol-name r)))))))
+                                             (ecb-string< (symbol-name l)
+                                                          (symbol-name r)))))))
     (append emacs-vars semantic-vars speedbar-vars
             ecb-internal-vars ecb-options)))
 
