@@ -26,7 +26,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-upgrade.el,v 1.62 2004/02/16 18:53:56 berndl Exp $
+;; $Id: ecb-upgrade.el,v 1.86 2004/08/31 15:33:52 berndl Exp $
 
 ;;; Commentary:
 ;;
@@ -159,7 +159,7 @@
 
 ;; IMPORTANT: The version-number is auto-frobbed from the Makefile. Do not
 ;; change it here!
-(defconst ecb-version "2.21"
+(defconst ecb-version "2.27"
   "Current ECB version.")
 
 (eval-when-compile
@@ -176,7 +176,28 @@
 
 ;; Each NEWS-string should be a one-liner shorter than 70 chars
 (defconst ecb-upgrade-news
-  '(("2.21" . ("Advice for `balance-windows' so only the edit-windows are balanced."
+  '(
+    ("2.27" . ("Much saver resizing-mechanism for permanent compile-window. See NEWS."))
+    ("2.26" . ("Some regexp-options has been changed to regexp-list-options. See NEWS."
+               "New option `ecb-history-exclude-file-regexps'."
+               "`ecb-expand-methods-nodes' works for non-semantic-buffers too."
+               "Readonly-sourcefiles are display in a different face."))
+    ("2.25" . ("`ecb-sort-history-items' has been renamed to `ecb-history-sort-method'"
+               "New options `ecb-sources-sort-ignore-case' and `ecb-history-sort-ignore-case'"
+               "New icons for parent-display in the Methods-buffer"))
+    ("2.24" . ("New \"current-type\"-filter for the Methods-buffer"
+               "Now directories are prescanned for emptyness"))
+    ("2.23" . ("New cedet1.0beta2 is supported."
+               "Distinction between functions and function-prototypes in the Methods-buffer"
+               "The command `ecb-toggle-layout' now has a prefix-argument"
+               "Default tag-filters for certain files which are applied automatically"
+               "Double-clicking the mouse-1-button now works with integrated speedbar"
+               "A new hook `ecb-speedbar-before-activate-hook'"))
+    ("2.22" . ("New nifty feature for filtering the tags displayed in the Methods-buffer"
+               "Much smarter mechanism to highlight the current tag in the methods-buffer"
+               "New option `ecb-auto-expand-tag-tree-collapse-other'."
+               "Fixed a bug preventing the native Windows-port of XEmacs from working."))
+    ("2.21" . ("Advice for `balance-windows' so only the edit-windows are balanced."
                "Gnus, BBDB, VM, Xrefactory etc. work even when ECB-windows are visible."
                "Commands using `Electric-pop-up-window' now work correctly with ECB."
                "Fixed some annoying bugs and one fatal bug."))
@@ -216,8 +237,6 @@
                                    ecb-upgrade-toggle-layout-sequence))
     (ecb-cache-directory-contents . (ecb-cache-directory-contents
                                      ecb-upgrade-cache-directory-contents))
-    (ecb-source-file-regexps . (ecb-source-file-regexps
-                                ecb-upgrade-source-file-regexps))
     (ecb-layout-always-operate-in-edit-window . (ecb-layout-always-operate-in-edit-window
                                                  ecb-upgrade-alway-operate-in-edit-window))
     (ecb-truncate-lines . (ecb-truncate-lines
@@ -246,7 +265,8 @@
     (ecb-post-process-semantic-tokenlist . (ecb-post-process-semantic-taglist
                                             ecb-upgrade-post-process-semantic-tokenlist))
     (ecb-show-only-positioned-tokens . (ecb-show-only-positioned-tags identity))
-    (ecb-show-tokens . (ecb-show-tags identity))
+    (ecb-show-tokens . (ecb-show-tags ecb-upgrade-show-tags))
+    (ecb-show-tags . (ecb-show-tags ecb-upgrade-show-tags))
     (ecb-highlight-token-with-point . (ecb-highlight-tag-with-point identity))
     (ecb-highlight-token-with-point-delay . (ecb-highlight-tag-with-point-delay identity))
     (ecb-token-visit-post-actions . (ecb-tag-visit-post-actions
@@ -257,7 +277,16 @@
                                           ecb-upgrade-post-process-semantic-taglist))
     (ecb-primary-mouse-jump-destination . (ecb-mouse-click-destination identity))
     (ecb-split-edit-window . (ecb-split-edit-window-after-start ecb-upgrade-split-edit-window))
-    (ecb-other-window-jump-behavior . (ecb-other-window-behavior ecb-upgrade-other-window-jump-behavior)))
+    (ecb-sort-history-items . (ecb-history-sort-method ecb-upgrade-sort-history-items))
+    (ecb-other-window-jump-behavior . (ecb-other-window-behavior ecb-upgrade-other-window-jump-behavior))
+    (ecb-excluded-directories-regexp . (ecb-excluded-directories-regexps
+                                        ecb-upgrade-excluded-directories-regexp))
+    (ecb-source-file-regexps . (ecb-source-file-regexps
+                                ecb-upgrade-source-file-regexps))
+    (ecb-exclude-parents-regexp . (ecb-exclude-parents-regexps
+                                   ecb-upgrade-exclude-parents-regexp))
+    (ecb-auto-expand-tag-tree-collapse-other . (ecb-auto-expand-tag-tree-collapse-other
+                                                ecb-upgrade-auto-expand-tag-tree-collapse-other)))
   "Alist of all options which should be upgraded for current ECB-version.
 There are several reasons why an option should be contained in this alist:
 a) An old option has just be renamed in current-ECB version but has still the
@@ -364,9 +393,6 @@ The car is the old option symbol and the cdr is a 2-element-list with:
                       (cons (nth 0 elem) (nth 1 elem))))
           old-val))
 
-(defun ecb-upgrade-source-file-regexps (old-val)
-  (list (cons ".*" old-val)))
-
 (defun ecb-upgrade-truncate-lines (old-val)
   (if old-val
       '(t t t t)
@@ -524,6 +550,30 @@ The car is the old option symbol and the cdr is a 2-element-list with:
     (ecb-option-get-value 'ecb-other-window-behavior
                           'standard-value)))
 
+(defun ecb-upgrade-show-tags (old-val)
+  (ecb-option-get-value 'ecb-show-tags
+                        'standard-value))
+
+(defun ecb-upgrade-sort-history-items (old-val)
+  (if old-val ecb-sources-sort-method))
+
+(defun ecb-upgrade-excluded-directories-regexp (old-val)
+  (list old-val))
+
+(defun ecb-upgrade-source-file-regexps (old-val)
+  (let ((l (copy-tree old-val)))
+    (dolist (elem l)
+      (setcdr elem (list (list (cadr elem)) (list (caddr elem)))))
+    l))
+    
+(defun ecb-upgrade-exclude-parents-regexp (old-val)
+  (if old-val (list old-val)))
+
+(defun ecb-upgrade-auto-expand-tag-tree-collapse-other (old-val)
+  (if old-val
+      'only-if-on-tag
+    nil))
+
 ;; ----------------------------------------------------------------------
 ;; internal functions. Dot change anything below this line
 ;; ----------------------------------------------------------------------
@@ -538,11 +588,24 @@ The car is the old option symbol and the cdr is a 2-element-list with:
   :group 'ecb-upgrade-internal
   :type 'string)
 
+(defun ecb-custom-file-writeable-p ()
+  (ignore-errors (file-writable-p (ecb-custom-file))))
+
+(defun ecb-customize-save-variable (option value)
+  ;; because the adviced version of `custom-save-all' do only all the special
+  ;; needed things if `ecb-minor-mode' is on we must temporally set here this
+  ;; variable to not nil because the that time this function is called this
+  ;; variable is still nil (will be first set to t if the ecb-activation can
+  ;; not fail).
+  (let ((ecb-minor-mode t))
+    (if (ecb-custom-file-writeable-p)
+        (customize-save-variable option value)
+      (customize-set-variable option value))))
 
 (defun ecb-option-set-default (option)
   "Save the ECB-option OPTION with current default value."
-  (customize-save-variable option
-                           (ecb-option-get-value option 'standard-value)))
+  (ecb-customize-save-variable option
+                               (ecb-option-get-value option 'standard-value)))
 
 (defun ecb-option-upgrade (old-option)
   "Upgrade the old ECB-option OLD-OPTION if the following conditions are ALL
@@ -588,7 +651,7 @@ result-list!"
       (when (not (equal new-value 'ecb-no-upgrade-conversion))
         ;; the old-value has been transformed successfully into the new type
         ;; so we can save it.
-        (customize-save-variable (nth 0 upgrade-elem) new-value))
+        (ecb-customize-save-variable (nth 0 upgrade-elem) new-value))
       ;; we return the value of the transforming-function even if it is
       ;; 'ecb-no-upgrade-conversion!
       (list new-value))))
@@ -610,7 +673,7 @@ after an ECB-upgrade.")
                     ecb-version))
     (setq ecb-old-ecb-version (ecb-option-get-value 'ecb-options-version
                                                     'saved-value))
-    (customize-save-variable 'ecb-options-version ecb-version)))
+    (ecb-customize-save-variable 'ecb-options-version ecb-version)))
   
 
 (defvar ecb-not-compatible-options nil
@@ -672,6 +735,7 @@ done then the option is reset to the default-value of current ECB-version."
           (ecb-option-set-default (car option)))))
     ;; Now we store the version of the options
     (ecb-store-current-options-version)))
+    
 
 (defvar ecb-renamed-options nil)
 
@@ -717,6 +781,13 @@ Note: This function upgrades only the renamed but not the incompatible options
   (if (or ecb-not-compatible-options ecb-renamed-options)
       (progn
         (with-output-to-temp-buffer "*ECB upgraded options*"
+          (when (and (or ecb-not-compatible-options ecb-renamed-options)
+                     (not (ecb-custom-file-writeable-p)))
+            (princ "Emacs can not save the upgraded options because the needed file\n")
+            (princ (if (ecb-custom-file)
+                       (concat (ecb-custom-file) " is not writeable!")
+                     "does not exist!"))
+            (princ "\nPlease ensure that the new values will be stored!\n\n"))
           (when ecb-not-compatible-options
             (princ "The values of the following options are incompatible with current type.\nECB has tried to transform the old-value to the new type. In cases where\nthis was not possible ECB has reset to the current default-value.")
             (princ "\n\n"))
@@ -842,13 +913,6 @@ options with their old \(before the upgrade/reset) and new values."
 ;; when cedet is stable (or a stable beta ;-) then we should add here a
 ;; cedet-required-version-min|max etc....
 
-;; (defconst ecb-required-semantic-version-min '(1 4 2 0))
-;; (defconst ecb-required-semantic-version-max '(2 0 3 9))
-;; (defconst ecb-required-eieio-version-min '(0 17 2 0))
-;; (defconst ecb-required-eieio-version-max '(0 18 3 9))
-;; (defconst ecb-required-speedbar-version-min '(0 14 1 1))
-;; (defconst ecb-required-speedbar-version-max '(0 15 3 9))
-
 (defvar ecb-all-requirements-available nil)
 
 (defun ecb-check-requirements (&optional just-check)
@@ -1163,11 +1227,11 @@ Return nil if ver-str has not the required syntax:
     (if (string-match "^\\([0-9]+\\)\\.\\([0-9]+\\)\\(beta\\|alpha\\|\\.\\)?\\([0-9]+\\)?$" str)
         (list (string-to-number (match-string 1 str))
               (string-to-number (match-string 2 str))
-              (if (string= (match-string 3 str) "alpha")
+              (if (ecb-string= (match-string 3 str) "alpha")
                   0
-                (if (string= (match-string 3 str) "beta")
+                (if (ecb-string= (match-string 3 str) "beta")
                     1
-                  (if (string= (match-string 3 str) ".")
+                  (if (ecb-string= (match-string 3 str) ".")
                       3
                     2)))
               (if (match-string 4 str)
@@ -1359,7 +1423,7 @@ current active version of PACKAGE."
                 (ecb-package-version-str2list curr-version))
               '(100 99 3 99))) ;; this version-number should be the biggest;-) 
         (install-dir nil))
-    (if (string= ver curr-version)
+    (if (ecb-string= ver curr-version)
         (ecb-error "You tried to download an already installed version %s - Stop!"
                    ver))
     (setq install-dir (ecb-package-download package ver url))
@@ -1378,7 +1442,7 @@ current active version of PACKAGE."
         (princ (concat "and then restarting Emacs the new version of "
                        package
                        " is loaded."))
-        (when (string= package "ecb")
+        (when (ecb-string= package "ecb")
           (princ "\n\nIf the value of `ecb-auto-compatibility-check' is not nil then the new version\n")
           (princ "checks at start-time if there are incompatible options! Please read the\n")
           (princ "documentation of this option!"))
@@ -1618,7 +1682,7 @@ for details about using \"wget\"."
           (princ "\n______________________________________________________________________________")
           (princ "\n\n")
           (princ "Please check the wget configuration in \"~/.wgetrc\" and also the value\n")
-          (princ (format "of the option %s." (if (string= package "ecb")
+          (princ (format "of the option %s." (if (ecb-string= package "ecb")
                                                  "`ecb-download-url'"
                                                "`ecb-cedet-url'")))
           (princ " ECB has tried to get informations from\nthe following URL:\n\n")
@@ -1662,8 +1726,8 @@ for details about using \"wget\"."
             )
         (if (get-buffer ecb-download-buffername)
             (kill-buffer ecb-download-buffername))
-        (ecb-delete-file downloaded-filename))
-
+        (ecb-delete-file downloaded-filename)
+        )
       ;; now we return the version-list
       version-list)))
 
