@@ -26,7 +26,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-util.el,v 1.97 2004/02/16 08:56:24 berndl Exp $
+;; $Id: ecb-util.el,v 1.111 2004/08/25 15:09:06 berndl Exp $
 
 ;;; Commentary:
 ;;
@@ -60,6 +60,9 @@
 (silentcomp-defvar noninteractive)
 (silentcomp-defun window-edges)
 (silentcomp-defun buffer-local-value)
+(silentcomp-defun posn-point)
+(silentcomp-defun posn-window)
+(silentcomp-defun event-start)
 ;; XEmacs
 (silentcomp-defun mswindows-cygwin-to-win32-path)
 (silentcomp-defun make-dialog-box)
@@ -81,13 +84,18 @@
 (silentcomp-defun thing-boundaries)
 (silentcomp-defun thing-symbol)
 
+(silentcomp-defun custom-file)
 
 ;; Some constants
 (defconst ecb-running-xemacs (string-match "XEmacs\\|Lucid" emacs-version))
 (defconst ecb-running-emacs-21 (and (not ecb-running-xemacs)
                                     (> emacs-major-version 20)))
 
-(defconst ecb-directory-sep-char ?/)
+;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Test this with native
+;; Windows-XEmacs if it works with this change correct and also if it works
+;; without this change incorrect!
+(defconst ecb-directory-sep-char
+  (if ecb-running-xemacs directory-sep-char ?/))
 (defconst ecb-directory-sep-string (char-to-string ecb-directory-sep-char))
 
 (defconst ecb-temp-dir
@@ -125,6 +133,43 @@
            (display-images-p)
          window-system)))
 
+;; -------------------------------------------------------------------
+;; Tracing - currently not used because we use the trace.el library!
+;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Offer conveniant wrappers for the
+;; trace-function-background stuff so users can easily trace a set of
+;; ecb-functions if there occur problems where a backtrace can not be
+;; generated.
+
+(defvar ecb-trace-defun-enter t)
+(defvar ecb-trace-defun-leave t)
+
+(defsubst ecb-defun-trace (mode fcn)
+  (cond ((equal mode 'enter)
+         (and ecb-trace-defun-enter
+              (message "ECB-function %S entered!" fcn)))
+        ((equal mode 'leave)
+         (and ecb-trace-defun-leave
+              (message "ECB-function %S leaved!" fcn)))))
+
+(defmacro ecb-defun (name args docstring &rest body)
+  `(eval-and-compile
+     (defun ,name ,args
+       ,docstring
+       ,@(if (not (equal (caar body) 'interactive))
+             (append (list `(ecb-defun-trace 'enter (quote ,name)))
+                     (list `(prog1 (progn
+                                     ,@body)
+                              (ecb-defun-trace 'leave (quote ,name)))))
+           (append (list (car body))
+                   (list `(ecb-defun-trace 'enter (quote ,name)))
+                   (list `(prog1 (progn
+                                   ,@(cdr body))
+                            (ecb-defun-trace 'leave (quote ,name)))))))))
+
+
+;; -------------------------------------------------------------------
+
+
 ;; ---------- compatibility between GNU Emacs and XEmacs ---------------------
 
 ;; miscellaneous differences
@@ -132,6 +177,8 @@
 (if ecb-running-xemacs
     (progn
 ;;; Compatibility
+      (defun ecb-facep (face)
+        (memq face (face-list)))
       (defun ecb-noninteractive ()
         "Return non-nil if running non-interactively, i.e. in batch mode."
         (noninteractive))
@@ -148,6 +195,9 @@ Unless optional argument INPLACE is non-nil, return a new string."
       (defalias 'ecb-frame-parameter 'frame-property)
       (defalias 'ecb-line-beginning-pos 'point-at-bol)
       (defalias 'ecb-line-end-pos 'point-at-eol)
+      (defalias 'ecb-event-window 'event-window)
+      (defalias 'ecb-event-point 'event-point)
+      (defalias 'ecb-event-buffer 'event-buffer)
       (defalias 'ecb-window-full-width 'window-full-width)
       (defalias 'ecb-window-full-height 'window-height)
       (defun ecb-frame-char-width (&optional frame)
@@ -161,14 +211,20 @@ Unless optional argument INPLACE is non-nil, return a new string."
                 (/ (nth 2 pix-edges) (ecb-frame-char-width))
                 (/ (nth 3 pix-edges) (ecb-frame-char-height))))))
       
+  (defalias 'ecb-facep 'facep)
   (defun ecb-noninteractive ()
     "Return non-nil if running non-interactively, i.e. in batch mode."
     noninteractive)
   (defalias 'ecb-subst-char-in-string 'subst-char-in-string)
-  (defalias 'ecb-thing-at-point 'thing-at-point)
   (defalias 'ecb-frame-parameter 'frame-parameter)
   (defalias 'ecb-line-beginning-pos 'line-beginning-position)
   (defalias 'ecb-line-end-pos 'line-end-position)
+  (defun ecb-event-window (event)
+    (posn-window (event-start event)))
+  (defun ecb-event-point (event)
+    (posn-point (event-start event)))
+  (defun ecb-event-buffer (event)
+    (window-buffer (ecb-event-window event)))
   (defun ecb-window-full-width (&optional window)
     (let ((edges (window-edges window)))
       (- (nth 2 edges) (nth 0 edges))))
@@ -238,6 +294,60 @@ Unless optional argument INPLACE is non-nil, return a new string."
 
 ;; ---------- End of compatibility between GNU Emacs and XEmacs -------------
   
+(if (fboundp 'compare-strings)
+    (defalias 'ecb-compare-strings 'compare-strings)
+  (defun ecb-compare-strings (str1 start1 end1 str2 start2 end2 &optional ignore-case)
+    "Compare the contents of two strings.
+In string STR1, skip the first START1 characters and stop at END1.
+In string STR2, skip the first START2 characters and stop at END2.
+END1 and END2 default to the full lengths of the respective strings.
+
+Case is significant in this comparison if IGNORE-CASE is nil.
+
+The value is t if the strings (or specified portions) match.
+If string STR1 is less, the value is a negative number N;
+  - 1 - N is the number of characters that match at the beginning.
+If string STR1 is greater, the value is a positive number N;
+  N - 1 is the number of characters that match at the beginning."
+    (or start1 (setq start1 0))
+    (or start2 (setq start2 0))
+    (setq end1 (if end1
+                   (min end1 (length str1))
+                 (length str1)))
+    (setq end2 (if end2
+                   (min end2 (length str2))
+                 (length str2)))
+    (let ((i1 start1)
+          (i2 start2)
+          result c1 c2)
+      (while (and (not result) (< i1 end1) (< i2 end2))
+        (setq c1 (aref str1 i1)
+              c2 (aref str2 i2)
+              i1 (1+ i1)
+              i2 (1+ i2))
+        (if ignore-case
+            (setq c1 (upcase c1)
+                  c2 (upcase c2)))
+        (setq result (cond ((< c1 c2) (- i1))
+                           ((> c1 c2) i1))))
+      (or result
+          (cond ((< i1 end1) (1+ (- i1 start1)))
+                ((< i2 end2) (1- (- start1 i1)))
+                (t)))
+      )))
+
+(defsubst ecb-string= (str1 str2 &optional ignore-case)
+  (let ((s1 (or (and (stringp str1) str1) (symbol-name str1)))
+        (s2 (or (and (stringp str2) str2) (symbol-name str2))))
+    (eq (ecb-compare-strings s1 nil nil s2 nil nil ignore-case) t)))
+
+(defsubst ecb-string< (str1 str2 &optional ignore-case)
+  (let ((s1 (or (and (stringp str1) str1) (symbol-name str1)))
+        (s2 (or (and (stringp str2) str2) (symbol-name str2)))
+        (result nil))
+    (setq result (ecb-compare-strings s1 nil nil s2 nil nil ignore-case))
+    (and (numberp result) (< result 0))))
+
 ;; Emacs 20 has no window-list function and the XEmacs and Emacs 21 one has no
 ;; specified ordering. The following one is stolen from XEmacs and has fixed
 ;; this lack of a well defined order. We preserve also point of current
@@ -285,6 +395,12 @@ means not to count the minibuffer even if it is active."
         ;; of this function
         (goto-char current-point)))))
 
+(defun ecb-canonical-windows-list ()
+  "Return a list of all current visible windows in the `ecb-frame' \(starting
+  from the left-most top-most window) in the order `other-window' would walk
+  through these windows."
+  (ecb-window-list ecb-frame 0 (frame-first-window ecb-frame)))
+
 
 ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Attention. Current mechanism of
 ;; (de)activating the basic advices and the intelligent window advices of
@@ -302,10 +418,6 @@ means not to count the minibuffer even if it is active."
                                             (scroll-other-window . around)
                                             (custom-save-all . around)
                                             (count-windows . around)
-                                            (narrow-to-region . before)
-                                            (narrow-to-defun . before)
-                                            (narrow-to-page . before)
-                                            (widen . before)
                                             (scroll-all-mode . after))
                                         '((delete-frame . around)
                                           (compilation-set-window-height . around)
@@ -321,10 +433,6 @@ means not to count the minibuffer even if it is active."
                                           (tmm-prompt . around)
                                           (scroll-other-window . around)
                                           (custom-save-all . around)
-                                          (narrow-to-region . before)
-                                          (narrow-to-defun . before)
-                                          (narrow-to-page . before)
-                                          (widen . before)
                                           (count-windows . around)
                                           (scroll-all-mode . after)))
   "These functions are always adviced if ECB is active. Each element of the
@@ -389,32 +497,61 @@ another frame than the `ecb-frame'."
 
 ;; some basic advices
 
+(defun ecb-custom-file ()
+  "Filename of that file which is used by \(X)Emacs to store the
+customize-options."
+  (cond (ecb-running-xemacs
+         custom-file)
+        (ecb-running-emacs-21
+         (custom-file))
+        (t
+         (or custom-file
+             user-init-file))))
+
 (defadvice custom-save-all (around ecb)
   "Save the customized options completely in the background, i.e. the
 file-buffer where the value is saved \(see option `custom-file') is not parsed
 by semantic and also killed afterwards."
   (if ecb-minor-mode
-      (let ((ecb-window-sync nil)
+      (let (;; XEmacs 21.4 does not set this so we do it here, to ensure that
+            ;; the custom-file is loadede in an emacs-lisp-mode buffer, s.b.
+            (default-major-mode 'emacs-lisp-mode)
+            (ecb-window-sync nil)
             (kill-buffer-hook nil)
             ;; we prevent parsing the custom-file
             (semantic-before-toplevel-bovination-hook (lambda ()
                                                         nil))
+            (semantic--before-fetch-tags-hook (lambda ()
+                                                nil))
             (semantic-after-toplevel-cache-change-hook nil)
             (semantic-after-partial-cache-change-hook nil))
-        ;; now we do the standard task
-        ad-do-it
+        ;; Klaus Berndl <klaus.berndl@sdm.de>: we must ensure that the
+        ;; current-buffer has a lisp major-mode when the kernel of
+        ;; `custom-save-all' is called because cause of a bug (IMHO) in the
+        ;; `custom-save-delete' of GNU Emacs (which loads the file returned by
+        ;; `custom-file' with `default-major-mode' set to nil which in turn
+        ;; causes that new buffer will get the major-mode of the
+        ;; current-buffer) the file `custom-file' will get the major-mode of
+        ;; the current-buffer. So when the current-buffer has for example
+        ;; major-mode `c++-mode' then the file `custom-file' will be loaded
+        ;; into a buffer with major-mode c++-mode. The function
+        ;; `custom-save-delete' then parses this buffer with (forward-sexp
+        ;; (buffer-size)) which of course fails because forward-sexp tries to
+        ;; parse the custom-file (which is an emacs-lisp-file) as a c++-file
+        ;; with c++-paren-syntax.
+        ;; Solution: Ensure that the buffer *scratch* is current when calling
+        ;; custom-save-all so we have surely a lispy-buffer and therefore we
+        ;; can be sure that custom-file is loaded as lispy-buffer.
+        (save-excursion
+          (set-buffer (get-buffer-create "*scratch*"))
+          ;; now we do the standard task
+          ad-do-it)
         ;; now we have to kill the custom-file buffer otherwise semantic would
         ;; parse the buffer of custom-file and the method-buffer would be
         ;; updated with the contents of custom-file which is definitely not
         ;; desired.
         (ignore-errors
-          (kill-buffer (find-file-noselect (cond (ecb-running-xemacs
-                                                  custom-file)
-                                                 (ecb-running-emacs-21
-                                                  (custom-file))
-                                                 (t
-                                                  (or custom-file
-                                                      user-init-file)))))))
+          (kill-buffer (find-file-noselect (ecb-custom-file)))))
     ad-do-it))
 
 ;; assoc helpers
@@ -427,6 +564,7 @@ by semantic and also killed afterwards."
                                 elem)))
                   list)))
 
+
 (defun ecb-add-assoc (key-value list)
   (cons key-value list))
 
@@ -492,13 +630,81 @@ Return the sublist of LIST whose car is ITEM."
       (member item list)
     (memq item list)))
 
+(defsubst ecb-match-regexp-list (str regexp-list &optional elem-accessor
+                                     return-accessor)
+  "Return not nil if STR matches one of the regexps in REGEXP-LIST. If
+ELEM-ACCESSOR is a function then it is used to get the regexp from the
+processed elem of REGEXP-LIST. If nil the elem itself is used. If
+RETURN-ACCESSOR is a function then it is used to get from the matching elem
+the object to return. If nil then the matching elem itself is returned."
+  (let ((elem-acc (or elem-accessor 'identity))
+        (return-acc (or return-accessor 'identity)))
+    (catch 'exit
+      (dolist (elem regexp-list)
+        (let ((case-fold-search t))
+          (save-match-data
+            (if (string-match (funcall elem-acc elem) str)
+                (throw 'exit (funcall return-acc elem))))
+          nil)))))
+
+
 (defun ecb-set-elt (seq n val)
   "Set VAL as new N-th element of SEQ. SEQ can be any sequence. SEQ will be
-changed."
-  (if (listp seq) (setcar (nthcdr n seq) val) (aset seq n val)))
+changed because this is desctructive function. SEQ is returned."
+  (if (listp seq)
+      (setcar (nthcdr n seq) val)
+    (aset seq n val))
+  seq)
+
+(defun ecb-replace-first-occurence (seq old-elem new-elem)
+  "Replace in SEQ the first occurence of OLD-ELEM with NEW-ELEM. Comparison is
+done by `equal'. This is desctructive function. SEQ is returned."
+  (let ((pos (ecb-position seq old-elem)))
+    (if pos
+        (ecb-set-elt seq pos new-elem)))
+  seq)
+
+(defun ecb-replace-all-occurences (seq old-elem new-elem)
+  "Replace in SEQ all occurences of OLD-ELEM with NEW-ELEM. Comparison is
+done by `equal'. This is desctructive function. SEQ is returned."
+  (while (ecb-position seq old-elem)
+    (setq seq (ecb-replace-first-occurence seq old-elem new-elem)))
+  seq)
+
+(defun ecb-remove-first-occurence-from-list (list elem)
+  "Replace first occurence of ELEM from LIST. Comparison is done by `equal'.
+This is desctructive function. LIST is returned."
+  (delq nil (ecb-replace-first-occurence list elem nil)))
+
+(defun ecb-remove-all-occurences-from-list (list elem)
+  "Replace all occurences of ELEM from LIST. Comparison is done by `equal'.
+This is desctructive function. LIST is returned."
+  (delq nil
+        (progn          
+          (while (ecb-position list elem)
+            (setq list (ecb-replace-first-occurence list elem nil)))
+          list)))
 
 ;; canonical filenames
 
+(defun ecb-fix-path (path)
+  "Fixes an annoying behavior of the native windows-version of XEmacs:
+When PATH contains only a drive-letter and a : then `expand-file-name' does
+not interpret this PATH as root of that drive. So we add a trailing
+`directory-sep-char' and return this new path because then `expand-file-name'
+treats this as root-dir of that drive. For all \(X)Emacs-version besides the
+native-windows-XEmacs PATH is returned."
+  (if (and ecb-running-xemacs
+           (equal system-type 'windows-nt))
+      (if (and (= (length path) 2)
+               (equal (aref path 1) ?:))
+          (concat path ecb-directory-sep-string)
+        path)
+    path))
+
+;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: What about the new cygwin-version
+;; of GNU Emacs 21? We have to test if this function and all locations where
+;; `ecb-fix-path' is used work correctly with the cygwin-port of GNU Emacs.
 (defun ecb-fix-filename (path &optional filename substitute-env-vars)
   "Normalizes path- and filenames for ECB. If FILENAME is not nil its pure
 filename \(i.e. without directory part) will be concatenated to PATH. The
@@ -506,13 +712,27 @@ result will never end with the directory-separator! If SUBSTITUTE-ENV-VARS is
 not nil then in both PATH and FILENAME env-var substitution is done. If the
 `system-type' is 'cygwin32 then the path is converted to win32-path-style!"
   (when (stringp path)
-    (let (norm-path)    
-      (setq norm-path (if (and ecb-running-xemacs (equal system-type 'cygwin32))
-                          (mswindows-cygwin-to-win32-path path)
-                        path))
+    (let (norm-path)
+      (setq norm-path (if ecb-running-xemacs
+                          (cond ((equal system-type 'cygwin32)
+                                 (mswindows-cygwin-to-win32-path
+                                  (expand-file-name path)))
+                                ((equal system-type 'windows-nt)
+                                 (expand-file-name (ecb-fix-path path)))
+                                (t (expand-file-name path)))
+                        (expand-file-name path)))
+      ;; For windows systems we normalize drive-letters to downcase
+      (setq norm-path (if (and (member system-type '(windows-nt cygwin32))
+                               (> (length norm-path) 1)
+                               (equal (aref norm-path 1) ?:))
+                          (concat (downcase (substring norm-path 0 2))
+                                  (substring norm-path 2))
+                        norm-path))
+      ;; substitute environment-variables
       (setq norm-path (expand-file-name (if substitute-env-vars
                                             (substitute-in-file-name norm-path)
                                           norm-path)))
+      ;; delete a trailing directory-separator if there is any
       (setq norm-path (if (and (> (length norm-path) 1)
                                (= (aref norm-path
                                         (1- (length norm-path))) ecb-directory-sep-char))
@@ -576,7 +796,7 @@ height is that fraction of the frame."
     (error "Window is not alive!")))
 
 ;; stolen from query.el and slightly enhanced
-;; This for a small number of choices each of them a short string
+;; This is for a small number of choices each of them a short string
 (defun ecb-query-string (prompt choices &optional other-prompt)
   "Prints PROMPT and returns a string which must be one of CHOICES.
 CHOICES is either a list of strings whereas the first choice is the default
@@ -601,9 +821,9 @@ the user is prompted with OTHER-PROMPT to insert any arbitrary string."
                                      new-choices)
                              '('("" t))))
     (setq answer (completing-read prompt new-choices nil t))
-    (cond ((string= answer "")
+    (cond ((ecb-string= answer "")
            (setq answer default))
-          ((string= answer "other")
+          ((ecb-string= answer "other")
            (setq answer (read-string (concat other-prompt ": ")))))
     answer))
 
@@ -625,7 +845,7 @@ be made either with the mouse or with the keyboard."
                                   completion-list
                                   nil t
                                   (try-completion "" completion-list))))
-    (if (string= answer "")
+    (if (ecb-string= answer "")
         (car choices)
       answer)))
 
@@ -648,7 +868,7 @@ be made either with the mouse or with the keyboard."
                   (mapcar (function (lambda (x) (list x t)))
                           choices)
                   nil t)))
-    (if (string= answer "")
+    (if (ecb-string= answer "")
         (car choices)
       answer)))
 
@@ -671,7 +891,7 @@ be made either with the mouse or with the keyboard."
                  (mapcar (function (lambda (x) (list x t)))
                          choices)
                  nil t)))
-    (if (string= answer "")
+    (if (ecb-string= answer "")
         (car choices)
       answer)))
 
@@ -736,7 +956,7 @@ INIT-VALUE can be either a number or a string-representation of a number."
   (let ((init (cond ((numberp init-value)
                      (number-to-string init-value))
                     ((stringp init-value)
-                     (if (string= init-value "0")
+                     (if (ecb-string= init-value "0")
                          init-value
                        (if (not (= 0 (string-to-number init-value)))
                            init-value
@@ -745,7 +965,7 @@ INIT-VALUE can be either a number or a string-representation of a number."
         result)
     (while (progn
              (setq result (read-string prompt init))
-             (not (or (string= "0" result)
+             (not (or (ecb-string= "0" result)
                       (not (= 0 (string-to-number result)))))))
     (string-to-number result)))
 
@@ -799,34 +1019,49 @@ If `window-system' is nil then a simple message is displayed in the echo-area."
 (defun ecb-merge-face-into-text (text face)
   "Merge FACE to the already precolored TEXT so the values of all
 face-attributes of FACE take effect and but the values of all face-attributes
-of TEXT which are not set by FACE are preserved.
-For XEmacs this merge does currently not work therefore here FACE replaces all
-faces of TEXT!"
+of TEXT which are not set by FACE are preserved."
   (if (null face)
       text
-    (let ((newtext (concat text)))
-      (if ecb-running-xemacs
-          (add-text-properties 0 (length newtext) (list 'face face) newtext)
-        (alter-text-property 0 (length newtext) 'face
-                             (lambda (current-face)
-                               (let ((cf
-                                      (cond ((facep current-face)
-                                             (list current-face))
-                                            ((listp current-face)
-                                             current-face)
-                                            (t nil)))
-                                     (nf
-                                      (cond ((facep face)
-                                             (list face))
-                                            ((listp face)
-                                             face)
-                                            (t nil))))
-                                 ;; we must add the new-face in front of
-                                 ;; current-face to get the right merge!
-                                 (append nf cf)))
-                             newtext))
-      newtext)))
-
+    (if ecb-running-xemacs
+        (put-text-property 0 (length text) 'face
+                           (let* ((current-face (get-text-property 0
+                                                                   'face
+                                                                   text))
+                                  (cf
+                                   (cond ((ecb-facep current-face)
+                                          (list current-face))
+                                         ((listp current-face)
+                                          current-face)
+                                         (t nil)))
+                                  (nf
+                                   (cond ((ecb-facep face)
+                                          (list face))
+                                         ((listp face)
+                                          face)
+                                         (t nil))))
+                             ;; we must add the new-face in front of
+                             ;; current-face to get the right merge!
+                             (append nf cf))
+                           text)
+      (alter-text-property 0 (length text) 'face
+                           (lambda (current-face)
+                             (let ((cf
+                                    (cond ((ecb-facep current-face)
+                                           (list current-face))
+                                          ((listp current-face)
+                                           current-face)
+                                          (t nil)))
+                                   (nf
+                                    (cond ((ecb-facep face)
+                                           (list face))
+                                          ((listp face)
+                                           face)
+                                          (t nil))))
+                               ;; we must add the new-face in front of
+                               ;; current-face to get the right merge!
+                               (append nf cf)))
+                           text))
+    text))
 
 (defun ecb-error (&rest args)
   "Signals an error but prevents it from entering the debugger. This is
@@ -865,7 +1100,8 @@ with a single space-character."
   (let ((split-result (split-string str "^[\n\t ]*")))
     (or (or (and (cdr split-result) ;; GNU Emacs > 21.3
                  (car (cdr split-result)))
-            (car split-result)) "")))
+            (car split-result))
+        "")))
 
 (defun ecb-right-trim (str)
   "Return a string stripped of all trailing whitespaces of STR."
@@ -995,13 +1231,36 @@ Since it actually calls `start-process', not all features will work."
 	;; (if (not (sit-for timeout)) (read-event))
 	))))
 
-(defun ecb-position (list elem)
-  "Return the position of ELEM within LIST counting from 0. Comparison is done
+(defun ecb-position (seq elem)
+  "Return the position of ELEM within SEQ counting from 0. Comparison is done
 with `equal'."
-  (let ((pos (1- (length (member elem (reverse list))))))
-    (if (< pos 0)
-        nil
-      pos)))
+  (if (listp seq)
+      (let ((pos (- (length seq) (length (member elem seq)))))
+        (if (= pos (length seq))
+            nil
+          pos))
+    (catch 'found
+      (dotimes (i (length seq))
+        (if (equal elem (aref seq i))
+            (throw 'found i)))
+      nil)))
+
+(defun ecb-last (seq)
+  "Return the last elem of the sequence SEQ."
+  (if (listp seq)
+      (car (last seq))
+    (if seq
+        (aref seq (1- (length seq)))
+      nil)))
+
+(defun ecb-first (seq)
+  "Return the first elem of the sequence SEQ."
+  (if (listp seq)
+      (car seq)
+    (if seq
+        (aref seq 0)
+      nil)))
+  
 
 (defun ecb-next-listelem (list elem &optional nth-next)
   "Return that element of LIST which follows directly ELEM when ELEM is an
@@ -1022,6 +1281,24 @@ returned."
                    (length list))
               list))))
 
+(defun ecb-buffer-name (buffer-or-name)
+  "Return the buffer-name of BUFFER-OR-NAME."
+  (cond ((stringp buffer-or-name)
+         buffer-or-name)
+        ((bufferp buffer-or-name)
+         (buffer-name buffer-or-name))
+        (t
+         nil)))
+
+(defun ecb-buffer-obj (buffer-or-name)
+  "Return the buffer-object of BUFFER-OR-NAME."
+  (cond ((stringp buffer-or-name)
+         (get-buffer buffer-or-name))
+        ((bufferp buffer-or-name)
+         buffer-or-name)
+        (t
+         nil)))
+
 (defun ecb-file-content-as-string (file)
   "If FILE exists and is readable returns the contents as a string otherwise
 return nil.
@@ -1057,9 +1334,10 @@ nil whereas in the latter case the current-buffer is assumed."
                    (ecb-current-buffer-archive-extract-p))
                (ecb-current-buffer-archive-extract-p))))))
 
-(defun ecb-fit-str-to-width (str width)
-  "If STR is longer than WIDTH then fit it to WIDTH by stripping from left and
-prepend \"...\" to signalize that the string is stripped. If WIDTH >= length
+(defun ecb-fit-str-to-width (str width from)
+  "If STR is longer than WIDTH then fit it to WIDTH by stripping from left or
+right \(depends on FROM which can be 'left or 'right) and prepend \(rsp.
+append) \"...\" to signalize that the string is stripped. If WIDTH >= length
 of STR the always STR is returned. If either WIDTH or length of STR is < 5
 then an empty string is returned because stripping makes no sense here."
   (let ((len-str (length str)))
@@ -1068,7 +1346,10 @@ then an empty string is returned because stripping makes no sense here."
       (if (or (< len-str 5) ;; we want at least two characters visible of str
               (< width 5))
           ""
-        (concat "..." (substring str (* -1 (- width 3))))))))
+        (if (equal from 'left)
+            (concat "..." (substring str (* -1 (- width 3))))
+          (concat (substring str 0 (- width 3)) "..."))))))
+
 
 (defun ecb-make-windows-not-dedicated (&optional frame)
   "Make all windows of FRAME not dedicated."
@@ -1101,10 +1382,9 @@ the same ordering as `other-window' would walk through the frame."
                                   ;;   modeline via (:eval...) the new buffer
                                   ;;   scrolls autom. back to beginng of
                                   ;;   buffer.
-                                  ;; With the original window-list all is ok...
-                                  (ecb-window-list
-                                   ecb-frame 0
-                                   (frame-first-window ecb-frame))
+                                  ;; With the original window-list all is
+                                  ;; ok...
+                                  (ecb-canonical-windows-list)
                                 (window-list)))))))
 
 
@@ -1118,6 +1398,8 @@ buffer-local value in BUFFER then the global value of SYM is used."
           (set-buffer buffer)
           (symbol-value sym)))))
 
+
+
 ;; ringstuff
 
 (require 'ring)
@@ -1130,8 +1412,28 @@ buffer-local value in BUFFER then the global value of SYM is used."
 (defun ecb-ring-elements (ring)
   "Return a list of the lements of RING."
   (mapcar #'identity (cddr ring)))
-      
 
+(defvar ecb-max-submenu-depth 4
+  "The maximum depth of nesting submenus for the tree-buffers.")
+
+(defun ecb-create-menu-user-ext-type (curr-level max-level)
+  "Creates the :type-definition for the *-menu-user-extension options.
+This allows nested submenus for the popup-menus of the tree-buffers up to a
+maximum level of MAX-LEVEL. CURR-LEVEL must be 1 when used in a
+defcustom-clause and has to be <= MAX-LEVEL."
+  (list 'repeat (delq nil
+                      (list 'choice ':tag "Menu-entry" ':menu-tag "Menu-entry"
+                            ':value '(ignore "")
+                            (list 'const ':tag "Separator" ':value '("---"))
+                            (list 'list ':tag "Menu-command"
+                                  (list 'function ':tag "Function" ':value 'ignore)
+                                  (list 'string ':tag "Entry-name"))
+                            (if (= curr-level max-level)
+                                nil
+                              (list 'cons ':tag "Submenu"
+                                    (list 'string ':tag "Submenu-title")
+                                    (ecb-create-menu-user-ext-type (1+ curr-level)
+                                                                   max-level)))))))
 (silentcomp-provide 'ecb-util)
 
 ;;; ecb-util.el ends here
